# -*- coding: utf-8 -*-
import os
import unittest
import pep8

BASE_PATH = os.path.dirname(os.path.dirname(
    os.path.dirname(os.path.abspath(__file__))))


class PEP8Test(unittest.TestCase):
    def test_pep8(self):
        pep8.DEFAULT_EXCLUDE = '.tox,*.egg'
        pep8style = pep8.StyleGuide([['statistics', True],
                                     ['show-sources', True],
                                     ['repeat', True],
                                     ['paths', [BASE_PATH]]],
                                    parse_argv=False,
                                    config_file=True)
        report = pep8style.check_files()
        assert report.total_errors == 0



# -*- coding: utf-8 -*-
from django.core.urlresolvers import resolve
from django.test import TestCase
from django.contrib.auth.models import User
import shiori.bookmark.views
from shiori.bookmark.feeds import LatestEntries
import shiori_tests.tests.vars as v


class BookmarkTest(TestCase):

    def setUp(self):
        self.user = User.objects.create_user(v.username,
                                             v.email,
                                             v.password)

    def test_shiori_url_resolve(self):
        found = resolve('/shiori/')
        self.assertEqual(found.func, shiori.bookmark.views.index)

    def test_profile_url_resolve(self):
        found = resolve('/shiori/profile/')
        self.assertEqual(found.func, shiori.bookmark.views.profile)

    def test_add_url_resolve(self):
        found = resolve('/shiori/add/')
        self.assertEqual(found.func, shiori.bookmark.views.add)

    def test_categories_url_resolve(self):
        found = resolve('/shiori/categories')
        self.assertEqual(found.func, shiori.bookmark.views.categories)

    def test_category_url_resolve(self):
        found = resolve('/shiori/categories/dummy_id')
        self.assertEqual(found.func, shiori.bookmark.views.category)

    def test_tags_url_resolve(self):
        found = resolve('/shiori/tags')
        self.assertEqual(found.func, shiori.bookmark.views.tags)

    def test_tag_url_resolve(self):
        found = resolve('/shiori/tags/dummy_id')
        self.assertEqual(found.func, shiori.bookmark.views.tag)

    def test_feeds_url_resolve(self):
        response = self.client.get('/shiori/feeds')
        self.assertEqual(response.status_code, 200)

    def test_bookmark_url_resolve(self):
        found = resolve('/shiori/b/dummy_id')
        self.assertEqual(found.func, shiori.bookmark.views.bookmark)

    def test_index_view(self):
        request = self.client.get('/shiori/')
        request.user = self.user
        response = shiori.bookmark.views.index(request)
        self.assertTrue('logout' in response.content)
        self.assertEqual(response.status_code, 200)

    def test_profile_view(self):
        request = self.client.get('/shiori/profile')
        request.user = self.user
        response = shiori.bookmark.views.profile(request)
        self.assertTrue('date joined' in response.content)
        self.assertEqual(response.status_code, 200)

    def test_categories_view(self):
        request = self.client.get('/shiori/categories')
        request.user = self.user
        response = shiori.bookmark.views.categories(request)
        self.assertTrue('categories_list' in response.content)
        self.assertEqual(response.status_code, 200)

    def test_category_view(self):
        request = self.client.get('/shiori/category/dummy_id')
        request.user = self.user
        response = shiori.bookmark.views.category(request, 'dummy_id')
        self.assertTrue('category_view' in response.content)
        self.assertEqual(response.status_code, 200)

    def test_tags_view(self):
        request = self.client.get('/shiori/tags')
        request.user = self.user
        response = shiori.bookmark.views.tags(request)
        self.assertTrue('tags_list' in response.content)
        self.assertEqual(response.status_code, 200)

    def test_tag_view(self):
        request = self.client.get('/shiori/tag/dummy_id')
        request.user = self.user
        response = shiori.bookmark.views.tag(request, 'dummy_id')
        self.assertTrue('tag_view' in response.content)
        self.assertEqual(response.status_code, 200)

    def test_bookmark_view(self):
        request = self.client.get('/shiori/b/dummy_id')
        request.user = self.user
        response = shiori.bookmark.views.bookmark(request, 'dummy_id')
        self.assertTrue('bookmark_view' in response.content)
        self.assertEqual(response.status_code, 200)



# -*- coding: utf-8 -*-
from django.core.urlresolvers import resolve
from django.test import TestCase
from django.contrib.auth.models import User
import unittest
import json

from shiori.api.views import CategoryViewSet
import shiori_tests.tests.vars as v


class APITest(TestCase):

    def setUp(self):
        User.objects.create_user(v.username, v.email, v.password)
        User.objects.create_user(v.username2, v.email2, v.password2)
        self.client.login(username=v.username, password=v.password)

    def test_api_categories_url_resolve(self):
        response = self.client.get('/v1/categories')
        self.assertEqual(response.status_code, 200)

    def test_api_tags_url_resolve(self):
        response = self.client.get('/v1/tags')
        self.assertEqual(response.status_code, 200)

    def test_api_bookmarks_url_resolve(self):
        response = self.client.get('/v1/bookmarks')
        self.assertEqual(response.status_code, 200)

    def test_api_bookmark_tags_url_resolve(self):
        response = self.client.get('/v1/bookmark_tags')
        self.assertEqual(response.status_code, 200)

    def test_post_tag(self):
        payload = {'tag': v.tag}
        response = self.client.post('/v1/tags',
                                    content_type='application/json',
                                    data=json.dumps(payload))
        self.assertEqual(response.status_code, 201)

    def test_get_tags(self):
        payload = {'tag': v.tag}
        self.client.post('/v1/tags',
                         content_type='application/json',
                         data=json.dumps(payload))
        response = self.client.get('/v1/tags')
        self.assertEqual(response.status_code, 200)

    def test_get_tag(self):
        payload = {'tag': v.tag}
        r = self.client.post('/v1/tags',
                             content_type='application/json',
                             data=json.dumps(payload))
        id = json.loads(r.content).get('id').decode('utf-8')
        response = self.client.get('/v1/tags/%s' % id)
        self.assertEqual(response.status_code, 200)

    def test_delete_tag(self):
        payload = {'tag': v.tag}
        r = self.client.post('/v1/tags',
                             content_type='application/json',
                             data=json.dumps(payload))
        id = json.loads(r.content).get('id').decode('utf-8')
        response = self.client.delete('/v1/tags/%s' % id)
        self.assertEqual(response.status_code, 204)

    def test_put_tag(self):
        payload = {'tag': v.tag}
        r = self.client.post('/v1/tags',
                             content_type='application/json',
                             data=json.dumps(payload))
        id = json.loads(r.content).get('id').decode('utf-8')
        payload2 = {'id': id,
                    'tag': v.tag2}
        response = self.client.put('/v1/tags/%s' % id,
                                   content_type='application/json',
                                   data=json.dumps(payload2))
        self.assertEqual(response.status_code, 200)

    def test_get_tags_by_anonymous(self):
        payload = {'tag': v.tag}
        self.client.post('/v1/tags',
                         content_type='application/json',
                         data=json.dumps(payload))

        self.client.logout()
        r = self.client.get('/v1/tags')
        self.assertEqual(r.status_code, 200)

        id = json.loads(r.content).get('results')[0].get('id').encode('utf-8')
        response = self.client.get('/v1/tags/%s' % id)
        self.assertEqual(response.status_code, 200)

    def test_post_tags_by_anonymous(self):
        self.client.logout()
        payload = {'tag': v.tag}
        response = self.client.post('/v1/tags',
                                    content_type='application/json',
                                    data=json.dumps(payload))
        self.assertEqual(response.status_code, 403)

    def test_delete_tag_by_anonymous(self):
        payload = {'tag': v.tag}
        r = self.client.post('/v1/tags',
                             content_type='application/json',
                             data=json.dumps(payload))
        id = json.loads(r.content).get('id').decode('utf-8')

        self.client.logout()
        response = self.client.delete('/v1/tags/%s' % id)
        self.assertEqual(response.status_code, 403)

    def test_put_tag_by_anonymous(self):
        payload = {'tag': v.tag}
        r = self.client.post('/v1/tags',
                             content_type='application/json',
                             data=json.dumps(payload))
        id = json.loads(r.content).get('id').decode('utf-8')

        self.client.logout()
        payload2 = {'id': id,
                    'tag': v.tag2}
        response = self.client.put('/v1/tags/%s' % id,
                                   content_type='application/json',
                                   data=json.dumps(payload2))
        self.assertEqual(response.status_code, 403)

    # ToDo: delete is allowed by owner user only.
    @unittest.skip("ToDo skipping")
    def test_delete_tag_by_another_user(self):
        payload = {'tag': v.tag}
        r = self.client.post('/v1/tags',
                             content_type='application/json',
                             data=json.dumps(payload))
        id = json.loads(r.content).get('id').decode('utf-8')

        self.client.logout()
        self.client.login(username=v.username2, password=v.password2)
        response = self.client.delete('/v1/tags/%s' % id)
        self.assertEqual(response.status_code, 403)

    # ToDo: put is allowed by owner user only.
    @unittest.skip("ToDo skipping")
    def test_put_tag_by_another_user(self):
        payload = {'tag': v.tag}
        r = self.client.post('/v1/tags',
                             content_type='application/json',
                             data=json.dumps(payload))
        id = json.loads(r.content).get('id').decode('utf-8')

        self.client.logout()
        self.client.login(username=v.username2, password=v.password2)
        payload2 = {'id': id, 'tag': v.tag2}
        response = self.client.put('/v1/tags/%s' % id,
                                   content_type='application/json',
                                   data=json.dumps(payload2))
        self.assertEqual(response.status_code, 403)

    def test_post_category(self):
        payload = {'category': v.category}
        response = self.client.post('/v1/categories',
                                    content_type='application/json',
                                    data=json.dumps(payload))
        self.assertEqual(response.status_code, 201)

    def test_delete_category(self):
        payload = {'category': v.category}
        r = self.client.post('/v1/categories',
                             content_type='application/json',
                             data=json.dumps(payload))
        id = json.loads(r.content).get('id').decode('utf-8')
        response = self.client.delete('/v1/categories/%s' % id)
        self.assertEqual(response.status_code, 204)

    def test_put_category(self):
        payload = {'category': v.category}
        r = self.client.post('/v1/categories',
                             content_type='application/json',
                             data=json.dumps(payload))
        id = json.loads(r.content).get('id').decode('utf-8')
        payload2 = {'id': id,
                    'category': v.category2}
        response = self.client.put('/v1/categories/%s' % id,
                                   content_type='application/json',
                                   data=json.dumps(payload2))
        self.assertEqual(response.status_code, 200)

    def test_get_categories_by_anonymous(self):
        payload = {'category': v.category}
        self.client.post('/v1/categories',
                         content_type='application/json',
                         data=json.dumps(payload))

        self.client.logout()
        r = self.client.get('/v1/categories')
        self.assertEqual(r.status_code, 200)

        id = json.loads(r.content).get('results')[0].get('id').encode('utf-8')
        response = self.client.get('/v1/categories/%s' % id)
        self.assertEqual(response.status_code, 200)

    def test_post_category_by_anonymous(self):
        self.client.logout()
        payload = {'category': v.category}
        response = self.client.post('/v1/categories',
                                    content_type='application/json',
                                    data=json.dumps(payload))
        self.assertEqual(response.status_code, 403)

    def test_delete_category_by_anonymous(self):
        payload = {'category': v.category}
        r = self.client.post('/v1/categories',
                             content_type='application/json',
                             data=json.dumps(payload))
        id = json.loads(r.content).get('id').decode('utf-8')

        self.client.logout()
        response = self.client.delete('/v1/categories/%s' % id)
        self.assertEqual(response.status_code, 403)

    def test_put_category_by_anonymous(self):
        payload = {'category': v.category}
        r = self.client.post('/v1/categories',
                             content_type='application/json',
                             data=json.dumps(payload))
        id = json.loads(r.content).get('id').decode('utf-8')

        self.client.logout()
        payload2 = {'id': id,
                    'category': v.category2}
        response = self.client.put('/v1/categories/%s' % id,
                                   content_type='application/json',
                                   data=json.dumps(payload2))
        self.assertEqual(response.status_code, 403)

    # ToDo: delete is allowed by owner user only.
    @unittest.skip("ToDo skipping")
    def test_delete_category_by_another_user(self):
        payload = {'category': v.category}
        r = self.client.post('/v1/categories',
                             content_type='application/json',
                             data=json.dumps(payload))
        id = json.loads(r.content).get('id').decode('utf-8')

        self.client.logout()
        self.client.login(username=v.username2, password=v.password2)
        response = self.client.delete('/v1/categories/%s' % id)
        self.assertEqual(response.status_code, 403)

    # ToDo: put is allowed by owner user only.
    @unittest.skip("ToDo skipping")
    def test_put_category_by_another_user(self):
        payload = {'category': v.category}
        r = self.client.post('/v1/categories',
                             content_type='application/json',
                             data=json.dumps(payload))
        id = json.loads(r.content).get('id').decode('utf-8')

        self.client.logout()
        self.client.login(username=v.username2, password=v.password2)
        payload2 = {'id': id, 'category': v.category2}
        response = self.client.put('/v1/categories/%s' % id,
                                   content_type='application/json',
                                   data=json.dumps(payload2))
        self.assertEqual(response.status_code, 403)

    def test_post_bookmark_without_category(self):
        payload = {'url': v.url,
                   'title': v.title,
                   'category': '',
                   'description': '',
                   'is_hide': False}
        response = self.client.post('/v1/bookmarks',
                                    content_type='application/json',
                                    data=json.dumps(payload))
        self.assertEqual(response.status_code, 400)

    def test_post_bookmark(self):
        payload = {'category': v.category}
        self.client.post('/v1/categories',
                         content_type='application/json',
                         data=json.dumps(payload))

        payload2 = {'url': v.url,
                    'title': v.title,
                    'category': v.category,
                    'description': '',
                    'is_hide': False}
        response = self.client.post('/v1/bookmarks',
                                    content_type='application/json',
                                    data=json.dumps(payload2))
        self.assertEqual(response.status_code, 201)

    def test_delete_bookmark(self):
        payload = {'category': v.category}
        self.client.post('/v1/categories',
                         content_type='application/json',
                         data=json.dumps(payload))

        payload2 = {'url': v.url,
                    'title': v.title,
                    'category': v.category,
                    'description': '',
                    'is_hide': False}
        r = self.client.post('/v1/bookmarks',
                             content_type='application/json',
                             data=json.dumps(payload2))
        id = json.loads(r.content).get('id').decode('utf-8')
        response = self.client.delete('/v1/bookmarks/%s' % id)
        self.assertEqual(response.status_code, 204)

    def test_put_bookmark(self):
        payload = {'category': v.category}
        self.client.post('/v1/categories',
                         content_type='application/json',
                         data=json.dumps(payload))
        payload2 = {'category': v.category2}
        self.client.post('/v1/categories',
                         content_type='application/json',
                         data=json.dumps(payload2))

        payload3 = {'url': v.url,
                    'title': v.title,
                    'category': v.category,
                    'description': '',
                    'is_hide': False}
        r = self.client.post('/v1/bookmarks',
                             content_type='application/json',
                             data=json.dumps(payload3))
        id = json.loads(r.content).get('id').decode('utf-8')
        payload4 = {'url': v.url,
                    'title': v.title,
                    'category': v.category2,
                    'description': v.description,
                    'is_hide': True}
        response = self.client.put('/v1/bookmarks/%s' % id,
                                   content_type='application/json',
                                   data=json.dumps(payload4))
        self.assertEqual(response.status_code, 200)

    def test_post_bookmark_tags(self):
        payload = {'category': v.category}
        self.client.post('/v1/categories',
                         content_type='application/json',
                         data=json.dumps(payload))

        payload2 = {'url': v.url,
                    'title': v.title,
                    'category': v.category,
                    'description': '',
                    'is_hide': False}
        self.client.post('/v1/bookmarks',
                         content_type='application/json',
                         data=json.dumps(payload2))

        payload3 = {'tag': v.tag}
        self.client.post('/v1/tags',
                         content_type='application/json',
                         data=json.dumps(payload3))

        payload4 = {'bookmark': v.url,
                    'tag': v.tag}

        response = self.client.post('/v1/bookmark_tags',
                                    content_type='application/json',
                                    data=json.dumps(payload4))
        self.assertEqual(response.status_code, 201)

    def test_delete_bookmark_tags(self):
        payload = {'category': v.category}
        self.client.post('/v1/categories',
                         content_type='application/json',
                         data=json.dumps(payload))

        payload2 = {'url': v.url,
                    'title': v.title,
                    'category': v.category,
                    'description': '',
                    'is_hide': False}
        self.client.post('/v1/bookmarks',
                         content_type='application/json',
                         data=json.dumps(payload2))

        payload3 = {'tag': v.tag}
        self.client.post('/v1/tags',
                         content_type='application/json',
                         data=json.dumps(payload3))

        payload4 = {'bookmark': v.url,
                    'tag': v.tag}

        r = self.client.post('/v1/bookmark_tags',
                             content_type='application/json',
                             data=json.dumps(payload4))

        id = json.loads(r.content).get('id')
        response = self.client.delete('/v1/bookmark_tags/%s' % id)
        self.assertEqual(response.status_code, 204)

    def test_put_bookmark_tags(self):
        payload = {'category': v.category}
        self.client.post('/v1/categories',
                         content_type='application/json',
                         data=json.dumps(payload))

        payload2 = {'url': v.url,
                    'title': v.title,
                    'category': v.category,
                    'description': '',
                    'is_hide': False}
        self.client.post('/v1/bookmarks',
                         content_type='application/json',
                         data=json.dumps(payload2))

        payload3 = {'tag': v.tag}
        self.client.post('/v1/tags',
                         content_type='application/json',
                         data=json.dumps(payload3))

        payload4 = {'tag': v.tag2}
        self.client.post('/v1/tags',
                         content_type='application/json',
                         data=json.dumps(payload4))

        payload4 = {'bookmark': v.url,
                    'tag': v.tag}

        r = self.client.post('/v1/bookmark_tags',
                             content_type='application/json',
                             data=json.dumps(payload4))

        id = json.loads(r.content).get('id')
        payload5 = {'id': id,
                    'bookmark': v.url,
                    'tag': v.tag2}

        response = self.client.put('/v1/bookmark_tags/%s' % id,
                                   content_type='application/json',
                                   data=json.dumps(payload5))
        self.assertEqual(response.status_code, 200)

    def test_post_bookmark_by_anonymous(self):
        payload = {'category': v.category}
        self.client.post('/v1/categories',
                         content_type='application/json',
                         data=json.dumps(payload))

        self.client.logout()
        payload2 = {'url': v.url,
                    'title': v.title,
                    'category': v.category,
                    'description': '',
                    'is_hide': False}
        response = self.client.post('/v1/bookmarks',
                                    content_type='application/json',
                                    data=json.dumps(payload2))
        self.assertEqual(response.status_code, 403)

    def test_delete_bookmark_by_anonymous(self):
        payload = {'category': v.category}
        self.client.post('/v1/categories',
                         content_type='application/json',
                         data=json.dumps(payload))

        payload2 = {'url': v.url,
                    'title': v.title,
                    'category': v.category,
                    'description': '',
                    'is_hide': False}
        r = self.client.post('/v1/bookmarks',
                             content_type='application/json',
                             data=json.dumps(payload2))
        id = json.loads(r.content).get('id').decode('utf-8')
        self.client.logout()
        response = self.client.delete('/v1/bookmarks/%s' % id)
        self.assertEqual(response.status_code, 403)

    def test_put_bookmark_by_anonymous(self):
        payload = {'category': v.category}
        self.client.post('/v1/categories',
                         content_type='application/json',
                         data=json.dumps(payload))
        payload2 = {'category': v.category2}
        self.client.post('/v1/categories',
                         content_type='application/json',
                         data=json.dumps(payload2))

        payload3 = {'url': v.url,
                    'title': v.title,
                    'category': v.category,
                    'description': '',
                    'is_hide': False}
        r = self.client.post('/v1/bookmarks',
                             content_type='application/json',
                             data=json.dumps(payload3))
        id = json.loads(r.content).get('id').decode('utf-8')
        payload4 = {'url': v.url,
                    'title': v.title,
                    'category': v.category2,
                    'description': v.description,
                    'is_hide': True}
        self.client.logout()
        response = self.client.put('/v1/bookmarks/%s' % id,
                                   content_type='application/json',
                                   data=json.dumps(payload4))
        self.assertEqual(response.status_code, 403)

    # ToDo: delete is allowed by owner user only.
    @unittest.skip("ToDo skipping")
    def test_delete_bookmark_by_another_user(self):
        payload = {'category': v.category}
        self.client.post('/v1/categories',
                         content_type='application/json',
                         data=json.dumps(payload))

        payload2 = {'url': v.url,
                    'title': v.title,
                    'category': v.category,
                    'description': '',
                    'is_hide': False}
        r = self.client.post('/v1/bookmarks',
                             content_type='application/json',
                             data=json.dumps(payload2))
        id = json.loads(r.content).get('id').decode('utf-8')
        self.client.logout()
        self.client.login(username=v.username2, password=v.password2)
        response = self.client.delete('/v1/bookmarks/%s' % id)
        self.assertEqual(response.status_code, 404)

        response = self.client.delete('/v1/bookmarks/%s?is_all=true' % id)
        self.assertEqual(response.status_code, 403)

    # ToDo: put is allowed by owner user only.
    # Url of bookmark is not unique in all users,
    # but url by users should be unique.
    @unittest.skip("ToDo skipping")
    def test_put_bookmark_by_another_user(self):
        payload = {'category': v.category}
        self.client.post('/v1/categories',
                         content_type='application/json',
                         data=json.dumps(payload))
        payload2 = {'category': v.category2}
        self.client.post('/v1/categories',
                         content_type='application/json',
                         data=json.dumps(payload2))

        payload3 = {'url': v.url,
                    'title': v.title,
                    'category': v.category,
                    'description': '',
                    'is_hide': False}
        r = self.client.post('/v1/bookmarks',
                             content_type='application/json',
                             data=json.dumps(payload3))
        id = json.loads(r.content).get('id').decode('utf-8')
        payload4 = {'url': v.url,
                    'title': v.title,
                    'category': v.category2,
                    'description': v.description,
                    'is_hide': True}
        self.client.logout()
        self.client.login(username=v.username2, password=v.password2)
        response = self.client.put('/v1/bookmarks/%s' % id,
                                   content_type='application/json',
                                   data=json.dumps(payload4))
        print(response.content)
        self.assertEqual(response.status_code, 403)

    def test_post_bookmark_tags_by_anonymous(self):
        payload = {'category': v.category}
        self.client.post('/v1/categories',
                         content_type='application/json',
                         data=json.dumps(payload))

        payload2 = {'url': v.url,
                    'title': v.title,
                    'category': v.category,
                    'description': '',
                    'is_hide': False}
        self.client.post('/v1/bookmarks',
                         content_type='application/json',
                         data=json.dumps(payload2))

        payload3 = {'tag': v.tag}
        self.client.post('/v1/tags',
                         content_type='application/json',
                         data=json.dumps(payload3))

        payload4 = {'bookmark': v.url,
                    'tag': v.tag}

        self.client.logout()
        response = self.client.post('/v1/bookmark_tags',
                                    content_type='application/json',
                                    data=json.dumps(payload4))
        self.assertEqual(response.status_code, 403)

    def test_delete_bookmark_tags_by_anonymous(self):
        payload = {'category': v.category}
        self.client.post('/v1/categories',
                         content_type='application/json',
                         data=json.dumps(payload))

        payload2 = {'url': v.url,
                    'title': v.title,
                    'category': v.category,
                    'description': '',
                    'is_hide': False}
        self.client.post('/v1/bookmarks',
                         content_type='application/json',
                         data=json.dumps(payload2))

        payload3 = {'tag': v.tag}
        self.client.post('/v1/tags',
                         content_type='application/json',
                         data=json.dumps(payload3))

        payload4 = {'bookmark': v.url,
                    'tag': v.tag}

        r = self.client.post('/v1/bookmark_tags',
                             content_type='application/json',
                             data=json.dumps(payload4))

        id = json.loads(r.content).get('id')
        self.client.logout()
        response = self.client.delete('/v1/bookmark_tags/%s' % id)
        self.assertEqual(response.status_code, 403)

    def test_put_bookmark_tags_by_anonymous(self):
        payload = {'category': v.category}
        self.client.post('/v1/categories',
                         content_type='application/json',
                         data=json.dumps(payload))

        payload2 = {'url': v.url,
                    'title': v.title,
                    'category': v.category,
                    'description': '',
                    'is_hide': False}
        self.client.post('/v1/bookmarks',
                         content_type='application/json',
                         data=json.dumps(payload2))

        payload3 = {'tag': v.tag}
        self.client.post('/v1/tags',
                         content_type='application/json',
                         data=json.dumps(payload3))

        payload4 = {'tag': v.tag2}
        self.client.post('/v1/tags',
                         content_type='application/json',
                         data=json.dumps(payload4))

        payload4 = {'bookmark': v.url,
                    'tag': v.tag}

        r = self.client.post('/v1/bookmark_tags',
                             content_type='application/json',
                             data=json.dumps(payload4))

        id = json.loads(r.content).get('id')
        payload5 = {'id': id,
                    'bookmark': v.url,
                    'tag': v.tag2}

        self.client.logout()
        response = self.client.put('/v1/bookmark_tags/%s' % id,
                                   content_type='application/json',
                                   data=json.dumps(payload5))
        self.assertEqual(response.status_code, 403)

    # ToDo: delete is allowed by owner user only.
    @unittest.skip("ToDo skipping")
    def test_delete_bookmark_tags_by_another_user(self):
        payload = {'category': v.category}
        self.client.post('/v1/categories',
                         content_type='application/json',
                         data=json.dumps(payload))

        payload2 = {'url': v.url,
                    'title': v.title,
                    'category': v.category,
                    'description': '',
                    'is_hide': False}
        self.client.post('/v1/bookmarks',
                         content_type='application/json',
                         data=json.dumps(payload2))

        payload3 = {'tag': v.tag}
        self.client.post('/v1/tags',
                         content_type='application/json',
                         data=json.dumps(payload3))

        payload4 = {'bookmark': v.url,
                    'tag': v.tag}

        r = self.client.post('/v1/bookmark_tags',
                             content_type='application/json',
                             data=json.dumps(payload4))

        id = json.loads(r.content).get('id')
        self.client.logout()
        self.client.login(username=v.username2, password=v.password2)
        response = self.client.delete('/v1/bookmark_tags/%s' % id)
        self.assertEqual(response.status_code, 403)

    # ToDo: put is allowed by owner user only.
    @unittest.skip("ToDo skipping")
    def test_put_bookmark_tags_by_another_user(self):
        payload = {'category': v.category}
        self.client.post('/v1/categories',
                         content_type='application/json',
                         data=json.dumps(payload))

        payload2 = {'url': v.url,
                    'title': v.title,
                    'category': v.category,
                    'description': '',
                    'is_hide': False}
        self.client.post('/v1/bookmarks',
                         content_type='application/json',
                         data=json.dumps(payload2))

        payload3 = {'tag': v.tag}
        self.client.post('/v1/tags',
                         content_type='application/json',
                         data=json.dumps(payload3))

        payload4 = {'tag': v.tag2}
        self.client.post('/v1/tags',
                         content_type='application/json',
                         data=json.dumps(payload4))

        payload4 = {'bookmark': v.url,
                    'tag': v.tag}

        r = self.client.post('/v1/bookmark_tags',
                             content_type='application/json',
                             data=json.dumps(payload4))

        id = json.loads(r.content).get('id')
        payload5 = {'id': id,
                    'bookmark': v.url,
                    'tag': v.tag2}

        self.client.logout()
        self.client.login(username=v.username2, password=v.password2)
        response = self.client.put('/v1/bookmark_tags/%s' % id,
                                   content_type='application/json',
                                   data=json.dumps(payload5))
        self.assertEqual(response.status_code, 403)



# -*- coding: utf-8 -*-
from django.contrib import admin
from shiori.bookmark.models import Bookmark, Category, Tag, BookmarkTag


admin.site.register(Bookmark)
admin.site.register(Category)
admin.site.register(Tag)
admin.site.register(BookmarkTag)



# -*- coding: utf-8 -*-
from rest_framework import serializers
from shiori.bookmark.models import Category, Tag, Bookmark, BookmarkTag


class CategorySerializer(serializers.ModelSerializer):

    class Meta:
        model = Category
        fields = ('id', 'category')


class TagSerializer(serializers.ModelSerializer):

    class Meta:
        model = Tag
        fields = ('id', 'tag')


class BookmarkSerializer(serializers.ModelSerializer):

    category = serializers.SlugRelatedField(many=False, slug_field='category')
    category_id = serializers.Field(source='category.id')
    tags = serializers.SlugRelatedField(many=True, slug_field='tag',
                                        read_only=True)
    owner = serializers.Field(source='owner.username')

    class Meta:
        model = Bookmark
        fields = ('id', 'url', 'title', 'category', 'category_id',
                  'tags', 'registered_datetime', 'description',
                  'owner', 'is_hide')


class BookmarkTagSerializer(serializers.ModelSerializer):

    bookmark = serializers.SlugRelatedField(many=False, slug_field='url')
    tag = serializers.SlugRelatedField(many=False, slug_field='tag')

    class Meta:
        model = BookmarkTag
        fields = ('id', 'bookmark', 'tag')
